@@ -71,7 +71,7 @@ static void usage()
 static void version()
 {
     printf("saphire version %s.\n", getenv("SAPHIRE_VERSION"));
-    printf("author Daisuke Minato(ab25cq@gmail.com).\n");
+    printf("author ab25cq(ab25cq@gmail.com).\n");
     
     exit(0);
 }
@@ -369,6 +369,19 @@ int main(int argc, char* argv[])
     readline_init(runtime_script);
     saphire_rehash();
 
+/*
+    minitscr();
+    int c = 0;
+    while(c != 'q') {
+        int meta;
+        c = mgetch(&meta);
+        printf("meta %d key %d\n", meta, c);
+        mendwin();
+    }
+    mreset_tty();
+    exit(1);
+*/
+
     /// ヒストリーを読み込む ///
     char* histfile = getenv("SAPHIRE_HISTFILE");
     if(histfile) {
