@@ -126,18 +126,9 @@ ALLOC void* CheckMemLeak_Malloc(size_t size, const char* file_name, int line, co
     
     entry = (sMallocEntry*)malloc(sizeof(sMallocEntry));
 
-    for(i=0; i<NAME_SIZE-1; i++) {
-         if(file_name[i]) entry->mFileName[i] = file_name[i];
-    }
-    entry->mFileName[i] = 0;
-   
+    xstrncpy(entry->mFileName, (char*)file_name, NAME_SIZE);
     entry->mLine = line;
-
-    for(i=0; i<NAME_SIZE-1; i++) {
-         if(func_name[i]) entry->mFuncName[i] = func_name[i];
-    }
-    entry->mFuncName[i] = 0;
-
+    xstrncpy(entry->mFuncName, (char*)func_name, NAME_SIZE);
     entry->mMemory = malloc(size);
    
     hash = (unsigned int)entry->mMemory % ARRAY_SIZE;
@@ -161,16 +152,16 @@ ALLOC void* CheckMemLeak_Realloc(void* memory, size_t size, const char* file_nam
     /// add new entry ///
     entry = (sMallocEntry*)malloc(sizeof(sMallocEntry));
       
-    memcpy(entry->mFileName, file_name, NAME_SIZE);
+    xstrncpy(entry->mFileName, (char*)file_name, NAME_SIZE);
     entry->mLine = line;
-    memcpy(entry->mFuncName, func_name, NAME_SIZE);
+    xstrncpy(entry->mFuncName, (char*)func_name, NAME_SIZE);
 
     entry->mMemory = realloc(memory, size);
     if(entry->mMemory == NULL) {
         fprintf(stderr,"false in allocating memory.");
         exit(1);
     }
-   
+
     hash = (unsigned int)entry->mMemory % ARRAY_SIZE;
     entry->mNextEntry = gMallocEntries[hash];
     gMallocEntries[hash] = entry;
@@ -186,8 +177,8 @@ ALLOC char* CheckMemLeak_Strdup(char* str, const char* file_name, int line, cons
     char* result;
 
     result = (char*)CheckMemLeak_Malloc(sizeof(char)*(strlen(str)+1), file_name, line, func_name);
-  
-    strcpy(result, str);
+
+    xstrncpy(result, str, strlen(str)+1);
 
     return result;
 }
