@@ -66,7 +66,55 @@ enum eCommandKind {
     kUnset,
     kUMask,
     kErrMsg,
-    kRunInfo, 
+    kRunInfo,
+
+    kConditionN,
+    kConditionZ,
+    kConditionB,
+    kConditionC,
+    kConditionD,
+    kConditionF,
+    kConditionH,
+    kConditionL,
+    kConditionP,
+    kConditionS2,
+    kConditionT,
+    kConditionG,
+    kConditionK,
+    kConditionU,
+    kConditionR,
+    kConditionW,
+    kConditionX,
+    kConditionO,
+    kConditionG2,
+    kConditionE,
+    kConditionS,
+    kConditionEq,
+    kConditionNEq,
+    kConditionSLT,
+    kConditionSGT,
+    kConditionSLE,
+    kConditionSGE,
+    kConditionSILT,
+    kConditionSIGT,
+    kConditionSILE,
+    kConditionSIGE,
+    kConditionEqI,
+    kConditionRE,
+    kConditionREI,
+    kConditionREIM,
+    kConditionREMI,
+    kConditionREM,
+    kConditionEQ,
+    kConditionNE,
+    kConditionLT,
+    kConditionLE,
+    kConditionGT,
+    kConditionGE,
+    kConditionNT,
+    kConditionOT,
+    kConditionEF,
+
     kInnerCommand,
     kCommand 
 };
@@ -475,7 +523,25 @@ void burry_stack_frame(vector_obj* fun_argv_before, string_obj** sname_strace, i
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
 ////////////////////////////////////////////////////////////
-int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, int j, char* title, BOOL input, BOOL* return_, BOOL enable_return, BOOL* break_, BOOL enable_break, char* sname, int line, vector_obj* envs, sObject* object, sFunction* running_fun, sClass* running_class);
+typedef struct {
+    //sStatments* statments;
+    //char* title
+    //sWFd* pipeout
+    //sRFd* pipein
+    //int pipeerr
+    BOOL* return_;
+    BOOL enable_return;
+    BOOL* break_;
+    BOOL enable_break;
+    vector_obj* parent_blocks;
+    //BOOL try_;
+    vector_obj* envs;
+    sObject* object;
+    sFunction* running_fun;
+    sClass* running_class;
+} sRunInfo;
+
+BOOL statment_tree_internal_commands_scan(sRunInfo* runinfo, vector_obj* argv, string_obj* field, BOOL input, sWFd* nextout, sRFd* nextin, int *rcode, BOOL ignore_case, BOOL multiline, char* sname, int sline, vector_obj* blocks, int nexterr, enum eKanjiCode code, enum eLineField lf);
 BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* sname, int line);
 
 ////////////////////////////////////////////////////////////
@@ -499,8 +565,9 @@ void tmpfile_sweep();
 extern vector_obj* gStackTraceFName;
 extern vector_obj* gStackTraceLineNum;
     // åե졼εϿ
+ 
 
-int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, BOOL enable_return, BOOL* break_ , BOOL enable_break, vector_obj* parent_blocks, BOOL try_, vector_obj* envs, sObject* object, sFunction* running_fun, sClass* running_class);
+int run(sRunInfo* runinfo, sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL try_);
     // saphire_compile2ǥѥ뤷ѥ̤¹Ԥ롣
 
 int saphire_shell4(char* command, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, sObject* object);
@@ -510,4 +577,13 @@ int saphire_shell5(sWFd* pipeout, char* command, char* title, sRFd* pipein, sObj
 void saphire_set_global_var2(char* name, char* value, sObject* object);
     // ֥Ȥ°Хѿꤹ
 
+typedef struct {
+    string_obj* mText;
+    BOOL mExpandEnv;
+} sHereDoc;
+
+sHereDoc* sHereDoc_new(MANAGED string_obj* text, BOOL expand_env);
+sHereDoc* sHereDoc_new2(sHereDoc* hdoc);
+void sHereDoc_delete(sHereDoc* self);
+
 #endif
