@@ -91,9 +91,53 @@ static BOOL read_until_next_close_character(char first_char, char last_char, cha
 
                     *read_end_of_statment = 1;
                 }
+                /// 変数展開 ///
                 else {
-                    string_push_back2(not_evaled, **p);
-                    (*p)++;
+                    BOOL quote;
+                    if(*(*p+1) == '$') {
+                        string_push_back2(not_evaled, **p); // $
+                        (*p)++;
+                        string_push_back2(not_evaled, **p); // $
+                        (*p)++;
+
+                        quote = FALSE;
+                    }
+                    else {
+                        string_push_back2(not_evaled, **p); // $
+                        (*p)++;
+
+                        quote = TRUE;
+                    }
+
+                    if(**p >= '0' && **p <= '9' || **p == '{' || **p >= 'a' && **p <= 'z' || **p >= 'A' && **p <= 'Z' || **p == '_') {
+                        if(**p == '{') {
+                            if(!read_until_next_close_character('{', '}', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
+                                return FALSE;
+                            }
+                        }
+                        else {
+                            while(**p) {
+                                if(**p >= '0' && **p <= '9' || **p >= 'a' && **p <= 'z' || **p >= 'A' && **p <= 'Z' || **p == '_') {
+                                    string_push_back2(not_evaled, **p); // 名前
+                                    (*p)++;
+                                }
+                                else {
+                                    break;
+                                }
+                            }
+                        }
+
+                        *read_end_of_statment = 1;
+                    }
+                    else {
+                        if(quote) {
+                            string_push_back2(not_evaled, '$');
+                        }
+                        else {
+                            string_push_back2(not_evaled, '$');
+                            string_push_back2(not_evaled, '$');
+                        }
+                    }
                 }
             }
             else if(squote || dquote) {
@@ -499,6 +543,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
 
                 *read_end_of_statment = 1;
             }
+            /// 変数展開
             else {
                 BOOL quote;
                 if(*(*p+1) == '$') {
@@ -1345,7 +1390,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                     else if(**p == ' ' || **p == '\t') {
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
-                            err_msg("invalid here document name", sname, *sline);
+                            err_msg("invalid here document name1", sname, *sline);
                             string_delete(name);
                             return FALSE;
                         }
@@ -1366,7 +1411,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         (*p)++;
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
-                            err_msg("invalid here document name", sname, *sline);
+                            err_msg("invalid here document name2", sname, *sline);
                             string_delete(name);
                             return FALSE;
                         }
@@ -1381,7 +1426,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         (*p)++;
                         skip_spaces(p, not_evaled);
                         if(**p != '\n') {
-                            err_msg("invalid here document name", sname, *sline);
+                            err_msg("invalid here document name3", sname, *sline);
                             string_delete(name);
                             return FALSE;
                         }
@@ -1390,20 +1435,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         break;
                     }
                     else {
-                        err_msg("invalid here document name", sname, *sline);
+                        err_msg("invalid here document name4", sname, *sline);
                         string_delete(name);
                         return FALSE;
                     }
                 }
 
-                if(dquote || squote) {
-                    err_msg("invalid here document name", sname, *sline);
-                    string_delete(name);
-                    return FALSE;
-                }
-                
-                if(string_c_str(name)[0] == 0) {
-                    err_msg("invalid here document name", sname, *sline);
+                if(dquote || squote|| string_c_str(name)[0] == 0) {
+                    err_msg("invalid here document name5", sname, *sline);
                     string_delete(name);
                     return FALSE;
                 }
@@ -1418,29 +1457,6 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         string_push_back2(text, '\n');
                         string_put(line, "");
                     }
-                    /// コマンド展開 ///
-                    else if(expand_env && **p == '$' && *(*p+1) == '(' || **p == '$' && *(*p+1) == '$' && *(*p+2) == '(') 
-                    {
-                        if(*(*p+1) == '$') {
-                            string_push_back2(not_evaled, **p); // $
-                            (*p)++;
-                            string_push_back2(not_evaled, **p); // $
-                            (*p)++;
-                        }
-                        else {
-                            string_push_back2(not_evaled, **p); // $
-                            (*p)++;
-                        }
-
-                        if(!read_until_next_close_character('(', ')', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
-                            string_delete(name);
-                            string_delete(line);
-                            string_delete(text);
-                            return FALSE;
-                        }
-
-                        *read_end_of_statment = 1;
-                    }
                     else {
                         string_push_back2(line, **p);
                         string_push_back2(not_evaled, **p);
@@ -1453,7 +1469,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                     }
                 }
 
-                vector_add(command->mArgs, ARG_NEW(7, MANAGED text));
+                vector_add(command->mArgs, ARG_NEW(7, sHereDoc_new(MANAGED text, expand_env)));
 
                 string_delete(line);
                 string_delete(name);
@@ -1696,7 +1712,7 @@ static BOOL read_statment(char**p, sStatment* statment, string_obj* not_evaled,
                 command->mKind = get_command_kind(string_c_str(arg->mBody));
             }
             else {
-                err_msg("invalid command name", sname, *sline);
+                err_msg("invalid command name6", sname, *sline);
                 sCommand_delete(command);
                 return FALSE;
             }
@@ -1745,7 +1761,6 @@ BOOL parse(char* cmdline, char* sname, int* sline, sStatments* statments)
             string_delete(not_evaled);
             return FALSE;
         }
-
         /// グローバルパイプ
         if(*p == '|' && *(p+1) >= '0' && *(p+1) <= '9' && *(p+2) == '>') {
             string_push_back(not_evaled, "|");
@@ -2142,6 +2157,114 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sO
 
             string_push_back2(out, *p++);
         }
+        /// ヘアドキュメントは飛ばす
+        else if(*p == '<' && *(p+1) == '<') {
+            string_push_back2(out, *p);
+            p++;
+            string_push_back2(out, *p);
+            p++;
+
+            skip_spaces(&p, out);
+
+            BOOL squote = FALSE;
+            BOOL dquote = FALSE;
+            if(*p == '\'') {
+                squote = TRUE;
+                string_push_back2(out, *p);
+                p++;
+            }
+            else if(*p == '"') {
+                dquote = TRUE;
+                string_push_back2(out, *p);
+                p++;
+            }
+
+            string_obj* name = STRING_NEW("");
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(out, *p);
+                    p++;
+                    break;
+                }
+                else if(*p == ' ' || *p == '\t') {
+                    skip_spaces(&p, out);
+                    if(*p != '\n') {
+                        err_msg("invalid here document name7", sname, *sline);
+                        string_delete(name);
+                        return FALSE;
+                    }
+                    string_push_back2(out, *p);
+                    p++;
+                    break;
+                }
+                else if(*p >='A' && * p <= 'Z' || *p >='a' && *p <='z')
+                {
+                    string_push_back2(name, *p);
+                    string_push_back2(out, *p);
+                    p++;
+                }
+                else if(dquote && *p == '"') {
+                    dquote = FALSE;
+                    string_push_back2(out, *p);
+                    p++;
+                    skip_spaces(&p, out);
+                    if(*p != '\n') {
+                        err_msg("invalid here document name8", sname, *sline);
+                        string_delete(name);
+                        return FALSE;
+                    }
+                    string_push_back2(out, *p);
+                    p++;
+                    break;
+                }
+                else if(squote && *p == '\'') {
+                    squote = FALSE;
+                    string_push_back2(out, *p);
+                    p++;
+                    skip_spaces(&p, out);
+                    if(*p != '\n') {
+                        err_msg("invalid here document name9", sname, *sline);
+                        string_delete(name);
+                        return FALSE;
+                    }
+                    string_push_back2(out, *p);
+                    p++;
+                    break;
+                }
+                else {
+                    err_msg("invalid here document name", sname, *sline);
+                    string_delete(name);
+                    return FALSE;
+                }
+            }
+
+            if(dquote || squote|| string_c_str(name)[0] == 0) {
+                err_msg("invalid here document name10", sname, *sline);
+                string_delete(name);
+                return FALSE;
+            }
+
+            string_obj* line = STRING_NEW("");
+            while(*p) {
+                if(*p == '\n') {
+                    string_push_back2(out, *p);
+                    p++;
+                }
+                else {
+                    string_push_back2(line, *p);
+                    string_push_back2(out, *p);
+                    p++;
+
+                    if(strstr(string_c_str(line), string_c_str(name)) == string_c_str(line))
+                    {
+                        break;
+                    }
+                }
+            }
+
+            string_delete(line);
+            string_delete(name);
+        }
 
         /// ブロック、サブシェル、条件式は飛ばす
         else if(*p == '(') {
@@ -2236,7 +2359,7 @@ BOOL expand_brace_expression(string_obj* tail, vector_obj* v, string_obj* start,
                         char buf[1024];
                         if(i < 0) {
                             buf[0] = '-';
-                            snprintf(buf + 1, 1024, buf2, abs(i));
+                            snprintf(buf + 1, 1023, buf2, abs(i));
                         }
                         else {
                             snprintf(buf, 1024, buf2, i);
@@ -2262,7 +2385,7 @@ BOOL expand_brace_expression(string_obj* tail, vector_obj* v, string_obj* start,
                         char buf[1024];
                         if(i < 0) {
                             buf[0] = '-';
-                            snprintf(buf + 1, 1024, buf2, abs(i));
+                            snprintf(buf + 1, 1023, buf2, abs(i));
                         }
                         else {
                             snprintf(buf, 1024, buf2, i);
@@ -2426,7 +2549,7 @@ BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname
                 string_push_back2(cmdline2, *p++);
             }
         }
-        /// 変数は虫
+        /// 変数は無視
         else if(*p == '$' && *(p+1) == '{') {
             string_push_back2(head, *p);
             string_push_back2(cmdline2, *p++);
@@ -2518,7 +2641,7 @@ BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname
                         char buf[1024];
                         if(i < 0) {
                             buf[0] = '-';
-                            snprintf(buf + 1, 1024, buf2, abs(i));
+                            snprintf(buf + 1, 1023, buf2, abs(i));
                         }
                         else {
                             snprintf(buf, 1024, buf2, i);
@@ -2546,7 +2669,7 @@ BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname
                         char buf[1024];
                         if(i < 0) {
                             buf[0] = '-';
-                            snprintf(buf + 1, 1024, buf2, abs(i));
+                            snprintf(buf + 1, 1023, buf2, abs(i));
                         }
                         else {
                             snprintf(buf, 1024, buf2, i);
@@ -2683,19 +2806,28 @@ int saphire_load_obj(char* fname, sWFd* pipeout, sRFd* pipein, int pipeerr, vect
     /// 実行 ///
     int return_ = FALSE;
     int break_ = FALSE;
-    vector_obj* envs = VECTOR_NEW(10);
-    int rcode = run(statments, fname
-                , pipeout, pipein, pipeerr
-                , &return_, TRUE, &break_, FALSE, parent_blocks, FALSE, envs, NULL, NULL, NULL);
+
+    sRunInfo runinfo;
+    runinfo.return_ = &return_;
+    runinfo.break_ = &break_;
+    runinfo.envs = VECTOR_NEW(10);
+    runinfo.enable_return = TRUE;
+    runinfo.enable_break = FALSE;
+    runinfo.parent_blocks = parent_blocks;
+    runinfo.object = NULL;
+    runinfo.running_fun = NULL;
+    runinfo.running_class = NULL;
+
+    int rcode = run(&runinfo, statments, fname, pipeout, pipein, pipeerr , FALSE);
 
     /// 解放 ///
     close(fd);
 
     int i;
-    for(i=0; i<vector_size(envs); i++) {
-        string_delete(vector_item(envs, i));
+    for(i=0; i<vector_size(runinfo.envs); i++) {
+        string_delete(vector_item(runinfo.envs, i));
     }
-    vector_delete(envs);
+    vector_delete(runinfo.envs);
     sStatments_delete(statments);
 
     return rcode;
